# Copyright 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
test_kuryr_tempest_plugin
----------------------------------

Tests for `kuryr_tempest_plugin` module.
"""

from tempest.lib import decorators

from kuryr_tempest_plugin.tests import base


class TestKuryr_tempest_plugin(base.BaseAdminKuryrTest):

    @decorators.idempotent_id('14990abd-76b4-476e-add2-321437d8cec5')
    def test_something(self):
        pass



# Copyright 2015
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

service_option = cfg.BoolOpt("kuryr",
                             default=True,
                             help="Whether or not kuryr is expected to be "
                                  "available")

ports_pool_batch = cfg.IntOpt("ports_pool_batch",
                              default=10,
                              help="The size of pool batch when "
                                   "KURYR_USE_PORT_POOLS is enabled")
ports_pool_max = cfg.IntOpt("ports_pool_max",
                            default=0,
                            help="Maximum number of ports when "
                                 "KURYR_USE_PORT_POOLS is enabled")
ports_pool_min = cfg.IntOpt("ports_pool_min",
                            default=5,
                            help="Minimum number of ports when "
                                 "KURYR_USE_PORT_POOLS is enabled")

kuryr_k8s_opts = [
    cfg.BoolOpt("port_pool_enabled", default=False,
                help="Whether or not port pool feature is enabled"),
    cfg.IntOpt("lb_build_timeout", default=900,
               help="The max time (in seconds) it should take to create LB"),
    cfg.BoolOpt("namespace_enabled", default=False,
                help="Whether or not namespace handler and driver are "
                     "enabled"),
    cfg.BoolOpt("network_policy_enabled", default=False,
                help="Whether or not network policy handler and driver are "
                     "enabled"),
    cfg.BoolOpt("service_tests_enabled", default=True,
                help="Whether or not service tests will be running"),
    cfg.BoolOpt("containerized", default=False,
                help="Whether or not kuryr-controller and kuryr-cni are "
                     "containerized"),
    cfg.StrOpt("kube_system_namespace", default="kube-system",
               help="Namespace where kuryr-controllers and kuryr-cnis run"),
    cfg.BoolOpt("run_tests_serial", default=False,
                help="Whether or not test run serially or in parallel"),
    cfg.StrOpt("kubernetes_project_name", default="k8s",
               help="The OpenStack project name for Kubernetes"),
    cfg.BoolOpt("npwg_multi_vif_enabled", default=False,
                help="Whether or not NPWG multi-vif feature is enabled"),
    cfg.StrOpt("ocp_router_fip", default=None, help="OCP Router floating IP"),
    cfg.BoolOpt("kuryr_daemon_enabled", default=True, help="Whether or not "
                "kuryr-kubernetes is configured to run with kuryr-daemon"),
    cfg.BoolOpt("ap_ha", default=False,
                help='Whether or not A/P HA of kuryr-controller is enabled'),
    cfg.StrOpt("controller_deployment_name", default="kuryr-controller",
               help="Name of Kubernetes Deployment running kuryr-controller "
                    "Pods"),
    cfg.BoolOpt("test_udp_services", default=False,
                help="Whether or not service UDP tests will be running"),
    cfg.BoolOpt("multi_worker_setup", default=False, help="Whether or not we "
                "have a multi-worker setup"),
    cfg.BoolOpt("cloud_provider", default=False, help="Whether or not a "
                "cloud provider is set"),
]



# Copyright 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from tempest import config
from tempest.lib import decorators

from kuryr_tempest_plugin.tests.scenario import base

CONF = config.CONF


class TestKuryrDaemon(base.BaseKuryrScenarioTest):
    @classmethod
    def skip_checks(cls):
        super(TestKuryrDaemon, cls).skip_checks()
        if (not CONF.kuryr_kubernetes.containerized) or (
           not CONF.kuryr_kubernetes.kuryr_daemon_enabled):
                raise cls.skipException("Kuryr cni should be containerized "
                                        "and Kuryr Daemon should be enabled "
                                        "to run this test.")

    @decorators.idempotent_id('bddf5441-1244-a49d-a125-b5fd3fb111a7')
    def test_kuryr_cni_daemon(self):
        namespace = CONF.kuryr_kubernetes.kube_system_namespace
        kube_system_pods = self.get_pod_name_list(
            namespace=namespace)
        cmd = ['cat', '/proc/1/cmdline']

        for kuryr_pod_name in kube_system_pods:
            if kuryr_pod_name.startswith('kuryr-cni'):
                self.assertIn(
                    'kuryr-daemon --config-file',
                    self.exec_command_in_pod(kuryr_pod_name, cmd, namespace,
                                             container='kuryr-cni'))



# Copyright 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions

from kuryr_tempest_plugin.tests.scenario import base

LOG = logging.getLogger(__name__)
CONF = config.CONF


class TestCrossPingScenario(base.BaseKuryrScenarioTest):

    @classmethod
    def skip_checks(cls):
        super(TestCrossPingScenario, cls).skip_checks()
        if not CONF.network_feature_enabled.floating_ips:
            raise cls.skipException("Floating ips are not available")

    @decorators.idempotent_id('bddf5441-1244-449d-a125-b5fddfb1a1a8')
    def test_vm_pod_ping(self):

        pod_name, pod = self.create_pod()
        self.addCleanup(self.delete_pod, pod_name, pod)
        pod_fip = self.assign_fip_to_pod(pod_name)
        ssh_client, fip = self.create_vm_for_connectivity_test()

        # check connectivity from VM to Pod
        cmd = ("ping -c4 -w4 %s &> /dev/null; echo $?" %
               pod_fip['floatingip']['floating_ip_address'])

        try:
            result = ssh_client.exec_command(cmd)
            if result:
                msg = ('Failed while trying to ping. Could not ping '
                       'from host "%s" to "%s".' % (
                           fip['floating_ip_address'],
                           pod_fip['floatingip']['floating_ip_address']))
                LOG.error(msg)
            self.assertEqual('0', result.rstrip('\n'))
        except exceptions.SSHExecCommandFailed:
            LOG.error("Couldn't ping server")

    @decorators.idempotent_id('bddf5441-1244-449d-a125-b5fddfb1a1a8')
    def test_pod_vm_ping(self):
        _, fip = self.create_vm_for_connectivity_test()
        pod_name, pod = self.create_pod()
        self.addCleanup(self.delete_pod, pod_name, pod)

        # check connectivity from Pod to VM
        cmd = [
            "/bin/sh", "-c", "ping -c 4 {dst_ip}>/dev/null ; echo $?".format(
                dst_ip=fip['floating_ip_address'])]
        self.assertEqual(self.exec_command_in_pod(pod_name, cmd), '0')

    @decorators.idempotent_id('bddf5441-1244-449d-a125-b5fddfb1a2a9')
    def test_pod_pod_ping(self):
        pod_name_list = []
        for i in range(2):
            pod_name, pod = self.create_pod()
            self.addCleanup(self.delete_pod, pod_name, pod)
            pod_name_list.append(pod_name)

        pod_ip = self.get_pod_ip(pod_name_list[1])
        cmd = [
            "/bin/sh", "-c", "ping -c 4 {dst_ip}>/dev/null ; echo $?".format(
                dst_ip=pod_ip)]
        self.assertEqual(self.exec_command_in_pod(pod_name_list[0], cmd), '0')



# Copyright 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import time

from oslo_concurrency import lockutils
from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators

from kuryr_tempest_plugin.tests.scenario import base
from kuryr_tempest_plugin.tests.scenario import consts

LOG = logging.getLogger(__name__)
CONF = config.CONF


class TestPortPoolScenario(base.BaseKuryrScenarioTest):
    CONFIG_MAP_NAME = 'kuryr-config'
    CONF_TO_UPDATE = 'kuryr.conf'
    VIF_POOL_SECTION = 'vif_pool'

    @classmethod
    def skip_checks(cls):
        super(TestPortPoolScenario, cls).skip_checks()
        if not CONF.kuryr_kubernetes.namespace_enabled:
            raise cls.skipException('Namespace driver and handler must be '
                                    'enabled to run these tests')
        if not CONF.kuryr_kubernetes.port_pool_enabled:
            raise cls.skipException(
                "Port pool feature should be enabled to run these tests.")

    @classmethod
    def resource_setup(cls):
        super(TestPortPoolScenario, cls).resource_setup()
        cls.PORTS_POOL_DEFAULT_DICT = cls.get_config_map_ini_value(
            name=cls.CONFIG_MAP_NAME, conf_for_get=cls.CONF_TO_UPDATE,
            section=cls.VIF_POOL_SECTION, keys=[
                'ports_pool_batch', 'ports_pool_min', 'ports_pool_max',
                'ports_pool_update_frequency'])

    def get_subnet_id_for_ns(self, namespace_name):
        subnet_name = 'ns/' + namespace_name + '-subnet'
        subnets_list = self.os_admin.subnets_client.list_subnets()
        subnet_id = [n['id'] for n in subnets_list['subnets']
                     if n['name'] == subnet_name][0]
        return subnet_id

    @decorators.idempotent_id('bddf5441-1244-449d-a125-b5fddfb1a3aa')
    @lockutils.synchronized('port-pool-restarts')
    def test_port_pool(self):
        namespace_name, namespace = self.create_namespace()
        self.addCleanup(self.delete_namespace, namespace_name)
        subnet_id = self.get_subnet_id_for_ns(namespace_name)

        # check the original length of list of ports for new ns
        port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        # create a pod to test the port pool increase
        pod_name1, _ = self.create_pod(namespace=namespace_name,
                                       labels={'type': 'demo'})

        # port number should increase by ports_pool_batch value
        updated_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        LOG.info("New_port_list_num = {} while pool_batch_conf = {}".format(
            updated_port_list_num, self.PORTS_POOL_DEFAULT_DICT[
                'ports_pool_batch']))
        num_to_compare = updated_port_list_num - int(
            self.PORTS_POOL_DEFAULT_DICT['ports_pool_batch'])
        self.assertEqual(num_to_compare, port_list_num)

        # create additional pod
        self.create_pod(namespace=namespace_name,
                        affinity={'podAffinity': consts.POD_AFFINITY})

        # the port pool should stay the same
        updated2_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        self.assertEqual(updated_port_list_num, updated2_port_list_num)

        # to test the reload of the pools, we will also test the restart of the
        # kuryr-controller
        self.restart_kuryr_controller()

        port_list_num_after_restart = len(
            self.os_admin.ports_client.list_ports(
                fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        self.assertEqual(updated_port_list_num, port_list_num_after_restart,
                         "Number of Neutron ports on namespace %s subnet "
                         "changed after kuryr-controller "
                         "restart" % namespace_name)

        # create additional pod
        pod_name3, _ = self.create_pod(
            namespace=namespace_name,
            affinity={'podAffinity': consts.POD_AFFINITY})

        # the total number of ports should stay the same
        updated3_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        self.assertEqual(updated_port_list_num, updated3_port_list_num)

        # check connectivity between pods
        pod_ip = self.get_pod_ip(pod_name1, namespace=namespace_name)
        cmd = [
            "/bin/sh", "-c", "ping -c 4 {dst_ip}>/dev/null ; echo $?".format(
                dst_ip=pod_ip)]
        self.assertEqual(self.exec_command_in_pod(
            pod_name3, cmd, namespace=namespace_name), '0')

    @decorators.idempotent_id('bddd5441-1244-429d-a125-b55ddfb134a9')
    @lockutils.synchronized('port-pool-restarts')
    def test_port_pool_update(self):
        UPDATED_POOL_BATCH = 3

        # Check resources are created
        namespace_name, namespace = self.create_namespace()
        self.addCleanup(self.delete_namespace, namespace_name)
        subnet_id = self.get_subnet_id_for_ns(namespace_name)
        self.update_config_map_ini_section_and_restart(
            name=self.CONFIG_MAP_NAME,
            conf_to_update=self.CONF_TO_UPDATE,
            section=self.VIF_POOL_SECTION,
            ports_pool_max=0,
            ports_pool_batch=UPDATED_POOL_BATCH,
            ports_pool_min=1)
        self.addCleanup(
            self.update_config_map_ini_section_and_restart,
            self.CONFIG_MAP_NAME, self.CONF_TO_UPDATE, self.VIF_POOL_SECTION,
            ports_pool_batch=self.PORTS_POOL_DEFAULT_DICT['ports_pool_batch'],
            ports_pool_max=self.PORTS_POOL_DEFAULT_DICT['ports_pool_max'],
            ports_pool_min=self.PORTS_POOL_DEFAULT_DICT['ports_pool_min'])

        # check the original length of list of ports for new ns
        port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        # create a pod to test the port pool increase by updated value
        pod_name1, pod1 = self.create_pod(namespace=namespace_name,
                                          labels={'type': 'demo'})

        # port number should increase by updated ports_pool_batch value
        updated_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        num_to_compare = updated_port_list_num - UPDATED_POOL_BATCH
        self.assertEqual(num_to_compare, port_list_num)

        # create additional pod
        pod_name2, pod2 = self.create_pod(
            namespace=namespace_name,
            affinity={'podAffinity': consts.POD_AFFINITY})

        # the total number of ports should stay the same
        updated2_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        self.assertEqual(updated_port_list_num, updated2_port_list_num)

        # check connectivity between pods
        pod_ip = self.get_pod_ip(pod_name1, namespace=namespace_name)
        cmd = [
            "/bin/sh", "-c", "ping -c 4 {dst_ip}>/dev/null ; echo $?".format(
                dst_ip=pod_ip)]
        self.assertEqual(self.exec_command_in_pod(
            pod_name2, cmd, namespace=namespace_name), '0')

    @decorators.idempotent_id('bddd5441-1244-459d-a133-b56ddfb147a6')
    @lockutils.synchronized('port-pool-restarts')
    def test_port_pool_noop_update(self):
        KUBERNETES_SECTION = 'kubernetes'
        VIF_POOL_SECTION = 'vif_pool'

        # Check resources are created
        namespace_name, namespace = self.create_namespace()
        self.addCleanup(self.delete_namespace, namespace_name)
        subnet_id = self.get_subnet_id_for_ns(namespace_name)

        # Read the value of the drivers
        update_pools_vif_drivers = self.get_config_map_ini_value(
            name=self.CONFIG_MAP_NAME, conf_for_get=self.CONF_TO_UPDATE,
            section=VIF_POOL_SECTION,
            keys=['pools_vif_drivers'])['pools_vif_drivers']
        vif_pool_driver = self.get_config_map_ini_value(
            name=self.CONFIG_MAP_NAME, conf_for_get=self.CONF_TO_UPDATE,
            section=KUBERNETES_SECTION,
            keys=['vif_pool_driver'])['vif_pool_driver']

        if update_pools_vif_drivers:
            self.update_config_map_ini_section(
                name=self.CONFIG_MAP_NAME,
                conf_to_update=self.CONF_TO_UPDATE,
                section=VIF_POOL_SECTION,
                pools_vif_drivers='')
            self.addCleanup(
                self.update_config_map_ini_section,
                self.CONFIG_MAP_NAME, self.CONF_TO_UPDATE, VIF_POOL_SECTION,
                pools_vif_drivers=update_pools_vif_drivers)

        self.update_config_map_ini_section_and_restart(
            name=self.CONFIG_MAP_NAME,
            conf_to_update=self.CONF_TO_UPDATE,
            section=KUBERNETES_SECTION,
            vif_pool_driver='noop')
        self.addCleanup(
            self.update_config_map_ini_section_and_restart,
            self.CONFIG_MAP_NAME, self.CONF_TO_UPDATE, KUBERNETES_SECTION,
            vif_pool_driver=vif_pool_driver)

        # check the original length of list of ports for new ns
        port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        # create a pod to test the port pool increase by 1
        self.create_pod(namespace=namespace_name, labels={'type': 'demo'})

        # port number should increase by 1
        new_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])

        self.assertEqual(port_list_num+1, new_port_list_num)

        # update pools_vif_drivers and vif_pool_driver to the previous values
        if update_pools_vif_drivers:
            self.update_config_map_ini_section(
                name=self.CONFIG_MAP_NAME,
                conf_to_update=self.CONF_TO_UPDATE,
                section=VIF_POOL_SECTION,
                pools_vif_drivers='')

        self.update_config_map_ini_section_and_restart(
            name=self.CONFIG_MAP_NAME,
            conf_to_update=self.CONF_TO_UPDATE,
            section=KUBERNETES_SECTION,
            vif_pool_driver=vif_pool_driver)

        # check that everything works as before when returning back from noop
        # configuration for vif_pool_driver
        self.create_pod(namespace=namespace_name,
                        affinity={'podAffinity': consts.POD_AFFINITY})

        # port number should increase by default ports_pool_batch value
        updated_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        num_to_compare = updated_port_list_num - int(
            self.PORTS_POOL_DEFAULT_DICT['ports_pool_batch'])
        self.assertEqual(num_to_compare, new_port_list_num)

    @decorators.idempotent_id('bddd5441-1244-429d-a123-b55ddfb137a6')
    @lockutils.synchronized('port-pool-restarts')
    def test_port_pool_min_max_update(self):
        POOL_BATCH = 2
        POOL_MAX = 2
        POOL_MIN = 1

        # Check resources are created
        namespace_name, namespace = self.create_namespace()
        self.addCleanup(self.delete_namespace, namespace_name)
        subnet_id = self.get_subnet_id_for_ns(namespace_name)

        self.update_config_map_ini_section_and_restart(
            name=self.CONFIG_MAP_NAME,
            conf_to_update=self.CONF_TO_UPDATE,
            section=self.VIF_POOL_SECTION,
            ports_pool_max=POOL_MAX,
            ports_pool_batch=POOL_BATCH,
            ports_pool_min=POOL_MIN)
        self.addCleanup(
            self.update_config_map_ini_section_and_restart,
            self.CONFIG_MAP_NAME, self.CONF_TO_UPDATE, self.VIF_POOL_SECTION,
            ports_pool_batch=self.PORTS_POOL_DEFAULT_DICT['ports_pool_batch'],
            ports_pool_max=self.PORTS_POOL_DEFAULT_DICT['ports_pool_max'],
            ports_pool_min=self.PORTS_POOL_DEFAULT_DICT['ports_pool_min'])

        # check the original length of list of ports for new ns
        initial_ports_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        # create a pod to test the port pool increase by updated batch value
        pod_name1, pod1 = self.create_pod(namespace=namespace_name,
                                          labels={'type': 'demo'})

        # port number should increase by updated ports_pool_batch value
        updated_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        num_to_compare = updated_port_list_num - POOL_BATCH
        self.assertEqual(num_to_compare, initial_ports_num)

        # need to wait till ports_pool_update_frequency expires so new batch
        # creation could be executed in order to create additional pod
        time.sleep(int(
            self.PORTS_POOL_DEFAULT_DICT['ports_pool_update_frequency']))
        pod_name2, pod2 = self.create_pod(
            namespace=namespace_name,
            affinity={'podAffinity': consts.POD_AFFINITY})

        # the total number of ports should increase by 2 as there is only 1
        # port free and POOL_MIN=1, so new port batch will be created
        updated2_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        num_to_compare = updated2_port_list_num - 2 * POOL_BATCH
        self.assertEqual(num_to_compare, initial_ports_num)

        # create additional pod
        pod_name3, pod3 = self.create_pod(
            namespace=namespace_name,
            affinity={'podAffinity': consts.POD_AFFINITY})

        # the total number of ports should stay the same
        updated3_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        self.assertEqual(updated2_port_list_num, updated3_port_list_num)

        # check connectivity between pods
        pod_ip = self.get_pod_ip(pod_name1, namespace=namespace_name)
        cmd = [
            "/bin/sh", "-c", "ping -c 4 {dst_ip}>/dev/null ; echo $?".format(
                dst_ip=pod_ip)]
        self.assertEqual(self.exec_command_in_pod(
            pod_name2, cmd, namespace=namespace_name), '0')

        # delete all pods and make sure the number of new ports added during
        # pod creation is equal to POOL_MAX
        for pod_name in (pod_name1, pod_name2, pod_name3):
            self.delete_pod(pod_name, namespace=namespace_name)
        # timeout is needed as it takes time for ports to be deleted
        time.sleep(30)
        updated4_port_list_num = len(self.os_admin.ports_client.list_ports(
            fixed_ips='subnet_id=%s' % subnet_id)['ports'])
        LOG.info("Number of ports after pods deletion and timeout = {}".format(
            updated4_port_list_num))
        self.assertEqual(updated4_port_list_num - initial_ports_num, POOL_MAX)



# Copyright 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import kubernetes
import time

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from kuryr_tempest_plugin.tests.scenario import base

LOG = logging.getLogger(__name__)
CONF = config.CONF

TIMEOUT_PERIOD = 20


class TestNetworkPolicyScenario(base.BaseKuryrScenarioTest):

    @classmethod
    def skip_checks(cls):
        super(TestNetworkPolicyScenario, cls).skip_checks()
        if not CONF.kuryr_kubernetes.network_policy_enabled:
            raise cls.skipException('Network Policy driver and handler must '
                                    'be enabled to run this tests')

    @decorators.idempotent_id('24577a9b-1d29-409b-8b60-da3b49d776b1')
    def test_create_delete_network_policy(self):
        np = self.create_network_policy()
        LOG.debug("Creating network policy %s" % np)
        network_policy_name = np.metadata.name
        kuryr_netpolicy_crd_name = 'np-' + network_policy_name
        network_policies = self.list_network_policies()
        kuryrnetpolicies = ''
        while True:
            try:
                kuryrnetpolicies = self.get_kuryr_netpolicy_crds(
                    name=kuryr_netpolicy_crd_name)
                break
            except kubernetes.client.rest.ApiException:
                time.sleep(1)
                continue
        sg_id = kuryrnetpolicies['spec']['securityGroupId']
        sgs = self.list_security_groups(fields='id')
        sg_ids = [sg['id'] for sg in sgs]
        self.assertIn(network_policy_name, network_policies)
        self.assertEqual(kuryr_netpolicy_crd_name,
                         str(kuryrnetpolicies['metadata']['name']))
        self.assertIn(sg_id, sg_ids)
        self.delete_network_policy(network_policy_name)
        self.assertNotIn(network_policy_name, self.list_network_policies())
        while True:
            time.sleep(1)
            try:
                self.get_kuryr_netpolicy_crds(name=kuryr_netpolicy_crd_name)
            except kubernetes.client.rest.ApiException:
                break
        sgs_after = self.list_security_groups(fields='id')
        sg_ids_after = [sg['id'] for sg in sgs_after]
        self.assertNotIn(sg_id, sg_ids_after)

    @decorators.idempotent_id('44daf8fe-6a4b-4ca9-8685-97fb1f573e5e')
    def test_update_network_policy(self):
        """Update a Network Policy with a new pod selector

        This method creates a Network Policy with a specific pod selector
        and updates it with a new pod selector. The CRD should always have
        the same pod selector as the Policy.
        """

        match_labels = {'app': 'demo'}
        np = self.create_network_policy(match_labels=match_labels)
        self.addCleanup(self.delete_network_policy,
                        np.metadata.name)
        kuryr_netpolicy_crd_name = 'np-' + np.metadata.name
        kuryrnetpolicies = ''
        start = time.time()
        while time.time() - start < TIMEOUT_PERIOD:
            try:
                kuryrnetpolicies = self.get_kuryr_netpolicy_crds(
                    name=kuryr_netpolicy_crd_name)
                break
            except kubernetes.client.rest.ApiException:
                time.sleep(1)
                continue
        if not kuryrnetpolicies:
            raise lib_exc.TimeoutException()

        crd_pod_selector = kuryrnetpolicies['spec']['podSelector']
        self.assertEqual(crd_pod_selector.get('matchLabels'),
                         match_labels)

        match_labels = {'context': 'demo'}
        np = self.read_network_policy(np)
        np.spec.pod_selector.match_labels = match_labels
        np = self.update_network_policy(np)

        labels = {}
        start = time.time()
        label_key = match_labels.keys()[0]
        while time.time() - start < TIMEOUT_PERIOD:
            try:
                time.sleep(1)
                updated_knp = self.get_kuryr_netpolicy_crds(
                    name=kuryr_netpolicy_crd_name)
                crd_pod_selector = updated_knp['spec']['podSelector']
                labels = crd_pod_selector.get('matchLabels')
                if labels.get(label_key):
                    break
            except kubernetes.client.rest.ApiException:
                continue

        if not labels.get(label_key):
            raise lib_exc.TimeoutException()

        self.assertEqual(labels, match_labels)



# Copyright 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
POD_OUTPUT = 'HELLO! I AM ALIVE!!!'
HA_ENDPOINT_NAME = 'kuryr-controller'
POD_AFFINITY = {'requiredDuringSchedulingIgnoredDuringExecution': [
    {'labelSelector': {'matchExpressions': [
        {'operator': 'In', 'values': ['demo'], 'key': 'type'}]},
        'topologyKey': 'kubernetes.io/hostname'}]}



# Copyright 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import kubernetes
import requests
import time

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators

from kuryr_tempest_plugin.tests.scenario import base
from kuryr_tempest_plugin.tests.scenario import consts

LOG = logging.getLogger(__name__)
CONF = config.CONF


class TestNamespaceScenario(base.BaseKuryrScenarioTest):

    @classmethod
    def skip_checks(cls):
        super(TestNamespaceScenario, cls).skip_checks()
        if not CONF.kuryr_kubernetes.namespace_enabled:
            raise cls.skipException('Namespace driver and handler must be '
                                    'enabled to run this tests')

    @classmethod
    def setup_clients(cls):
        super(TestNamespaceScenario, cls).setup_clients()

    @decorators.idempotent_id('bddd5441-1244-429d-a125-b53ddfb132a9')
    def test_namespace(self):
        # Check resources are created
        namespace_name, namespace = self.create_namespace()

        existing_namespaces = [ns.metadata.name
                               for ns in self.list_namespaces().items]

        self.assertIn(namespace_name, existing_namespaces)

        subnet_name = 'ns/' + namespace_name + '-subnet'
        kuryr_net_crd_name = 'ns-' + namespace_name

        seen_subnets = self.os_admin.subnets_client.list_subnets()
        seen_subnet_names = [n['name'] for n in seen_subnets['subnets']]

        self.assertIn(subnet_name, seen_subnet_names)

        subnet_id = [n['id'] for n in seen_subnets['subnets']
                     if n['name'] == subnet_name]
        net_id = [n['network_id'] for n in seen_subnets['subnets']
                  if n['name'] == subnet_name]

        kuryr_net_crd = self.get_kuryr_net_crds(kuryr_net_crd_name)

        self.assertIn(kuryr_net_crd_name, kuryr_net_crd['metadata']['name'])
        self.assertIn(kuryr_net_crd['spec']['subnetId'], subnet_id)
        self.assertIn(kuryr_net_crd['spec']['netId'], net_id)

        # Check namespace pod connectivity
        pod_name, pod = self.create_pod(labels={"app": 'pod-label'},
                                        namespace=namespace_name)
        svc_name, _ = self.create_service(pod_label=pod.metadata.labels,
                                          spec_type='LoadBalancer',
                                          namespace=namespace_name)
        svc_service_ip = self.get_service_ip(service_name=svc_name,
                                             spec_type='LoadBalancer',
                                             namespace=namespace_name)
        self.wait_service_status(svc_service_ip,
                                 CONF.kuryr_kubernetes.lb_build_timeout)

        requests.get("http://{dst_ip}".format(dst_ip=svc_service_ip))

        # Check resources are deleted
        self._delete_namespace_resources(namespace_name, kuryr_net_crd,
                                         subnet_name)

    @decorators.idempotent_id('bdde5441-1b44-449d-a125-b5fdbfb1a2a9')
    def test_namespace_sg_isolation(self):
        # Check security group resources are created
        ns1_name, ns1 = self.create_namespace()
        ns2_name, ns2 = self.create_namespace()

        existing_namespaces = [ns.metadata.name
                               for ns in self.list_namespaces().items]

        self.assertIn(ns1_name, existing_namespaces)
        self.assertIn(ns2_name, existing_namespaces)
        self.assertIn('default', existing_namespaces)

        subnet_ns1_name = 'ns/' + ns1_name + '-subnet'
        subnet_ns2_name = 'ns/' + ns2_name + '-subnet'
        net_crd_ns1_name = 'ns-' + ns1_name
        net_crd_ns2_name = 'ns-' + ns2_name

        net_crd_ns1 = self.get_kuryr_net_crds(net_crd_ns1_name)
        net_crd_ns2 = self.get_kuryr_net_crds(net_crd_ns2_name)

        self.assertIn(net_crd_ns1_name, net_crd_ns1['metadata']['name'])
        self.assertIn(net_crd_ns2_name, net_crd_ns2['metadata']['name'])

        seen_sgs = self.list_security_groups()
        seen_sg_ids = [sg['id'] for sg in seen_sgs]

        self.assertIn(net_crd_ns1['spec']['sgId'], seen_sg_ids)
        self.assertIn(net_crd_ns2['spec']['sgId'], seen_sg_ids)

        # Create pods in different namespaces
        pod_ns1_name, pod_ns1 = self.create_pod(labels={"app": 'pod-label'},
                                                namespace=ns1_name)

        pod_ns2_name, pod_ns2 = self.create_pod(labels={"app": 'pod-label'},
                                                namespace=ns2_name)

        pod_nsdefault_name, pod_nsdefault = self.create_pod(
            labels={"app": 'pod-label'}, namespace='default')
        self.addCleanup(self.delete_pod, pod_nsdefault_name)

        # Check namespace pod connectivity and isolation
        pod_ns2_ip = self.get_pod_ip(pod_ns2_name, ns2_name)
        pod_nsdefault_ip = self.get_pod_ip(pod_nsdefault_name)

        # check connectivity from NS1 to default
        cmd = ["/bin/sh", "-c", "curl {dst_ip}:8080".format(
            dst_ip=pod_nsdefault_ip)]
        self.assertIn(consts.POD_OUTPUT,
                      self.exec_command_in_pod(pod_ns1_name, cmd, ns1_name))

        # check no connectivity from NS1 to NS2
        cmd = ["/bin/sh", "-c", "curl {dst_ip}:8080".format(
            dst_ip=pod_ns2_ip)]
        self.assertNotIn(consts.POD_OUTPUT,
                         self.exec_command_in_pod(pod_ns1_name, cmd, ns1_name))

        # check connectivity from default to NS2
        cmd = ["/bin/sh", "-c", "curl {dst_ip}:8080".format(
            dst_ip=pod_ns2_ip)]
        self.assertIn(consts.POD_OUTPUT,
                      self.exec_command_in_pod(pod_nsdefault_name, cmd))

        self._delete_namespace_resources(ns1_name, net_crd_ns1,
                                         subnet_ns1_name)
        self._delete_namespace_resources(ns2_name, net_crd_ns2,
                                         subnet_ns2_name)

    @decorators.idempotent_id('b43f5421-1244-449d-a125-b5fddfb1a2a9')
    def test_namespace_sg_svc_isolation(self):
        # Check security group resources are created
        ns1_name, ns1 = self.create_namespace()
        ns2_name, ns2 = self.create_namespace()

        existing_namespaces = [ns.metadata.name
                               for ns in self.list_namespaces().items]

        self.assertIn(ns1_name, existing_namespaces)
        self.assertIn(ns2_name, existing_namespaces)
        self.assertIn('default', existing_namespaces)

        subnet_ns1_name = 'ns/' + ns1_name + '-subnet'
        subnet_ns2_name = 'ns/' + ns2_name + '-subnet'
        net_crd_ns1_name = 'ns-' + ns1_name
        net_crd_ns2_name = 'ns-' + ns2_name

        net_crd_ns1 = self.get_kuryr_net_crds(net_crd_ns1_name)
        net_crd_ns2 = self.get_kuryr_net_crds(net_crd_ns2_name)

        self.assertIn(net_crd_ns1_name, net_crd_ns1['metadata']['name'])
        self.assertIn(net_crd_ns2_name, net_crd_ns2['metadata']['name'])

        seen_sgs = self.list_security_groups()
        seen_sg_ids = [sg['id'] for sg in seen_sgs]

        self.assertIn(net_crd_ns1['spec']['sgId'], seen_sg_ids)
        self.assertIn(net_crd_ns2['spec']['sgId'], seen_sg_ids)

        # Create pods and services in different namespaces
        pod_ns1_name, pod_ns1 = self.create_pod(labels={"app": 'pod-label'},
                                                namespace=ns1_name)
        svc_ns1_name, _ = self.create_service(
            pod_label=pod_ns1.metadata.labels, namespace=ns1_name)
        svc_ns1_ip = self.get_service_ip(service_name=svc_ns1_name,
                                         namespace=ns1_name)

        pod_ns2_name, pod_ns2 = self.create_pod(labels={"app": 'pod-label'},
                                                namespace=ns2_name)
        svc_ns2_name, _ = self.create_service(
            pod_label=pod_ns2.metadata.labels, namespace=ns2_name)
        svc_ns2_ip = self.get_service_ip(service_name=svc_ns2_name,
                                         namespace=ns2_name)

        # Wait for services to be ready
        self.wait_service_status(svc_ns1_ip,
                                 CONF.kuryr_kubernetes.lb_build_timeout)
        self.wait_service_status(svc_ns2_ip,
                                 CONF.kuryr_kubernetes.lb_build_timeout)

        pod_nsdefault_name, pod_nsdefault = self.create_pod(
            labels={"app": 'pod-label'}, namespace='default')
        self.addCleanup(self.delete_pod, pod_nsdefault_name)

        # Check namespace svc connectivity and isolation
        # check connectivity from NS1 pod to NS1 service
        cmd = ["/bin/sh", "-c", "curl {dst_ip}".format(
            dst_ip=svc_ns1_ip)]
        self.assertIn(consts.POD_OUTPUT,
                      self.exec_command_in_pod(pod_ns1_name, cmd, ns1_name))

        # check no connectivity from NS1 pod to NS2 service
        cmd = ["/bin/sh", "-c", "curl {dst_ip}".format(
            dst_ip=svc_ns2_ip)]
        self.assertNotIn(consts.POD_OUTPUT,
                         self.exec_command_in_pod(pod_ns1_name, cmd, ns1_name))

        # check connectivity from default pod to NS2 service
        cmd = ["/bin/sh", "-c", "curl {dst_ip}".format(
            dst_ip=svc_ns2_ip)]
        self.assertIn(consts.POD_OUTPUT,
                      self.exec_command_in_pod(pod_nsdefault_name, cmd))

        # Check resources are deleted
        self._delete_namespace_resources(ns1_name, net_crd_ns1,
                                         subnet_ns1_name)
        self._delete_namespace_resources(ns2_name, net_crd_ns2,
                                         subnet_ns2_name)

    @decorators.idempotent_id('bddd5441-1244-429d-a125-b53ddfb132a9')
    def test_host_to_namespace_connectivity(self):
        # Create namespace and pod and service in that namespace
        namespace_name, namespace = self.create_namespace()
        self.addCleanup(self.delete_namespace, namespace_name)
        # Check host to namespace pod and service connectivity
        pod_name, pod = self.create_pod(labels={"app": 'pod-label'},
                                        namespace=namespace_name)
        pod_ip = self.get_pod_ip(pod_name, namespace=namespace_name)
        svc_name, _ = self.create_service(pod_label=pod.metadata.labels,
                                          namespace=namespace_name)
        service_ip = self.get_service_ip(service_name=svc_name,
                                         namespace=namespace_name)
        self.wait_service_status(service_ip,
                                 CONF.kuryr_kubernetes.lb_build_timeout)
        # Check connectivity to pod and service in the namespace
        self.ping_ip_address(pod_ip)
        resp = requests.get("http://{dst_ip}".format(dst_ip=service_ip))
        self.assertEqual(resp.status_code, 200)

    def _delete_namespace_resources(self, namespace, net_crd, subnet):
        # Check resources are deleted
        self.delete_namespace(namespace)

        while True:
            time.sleep(1)
            try:
                self.get_kuryr_net_crds(net_crd['metadata']['name'])
            except kubernetes.client.rest.ApiException:
                break

        existing_namespaces = [ns.metadata.name
                               for ns in self.list_namespaces().items]
        self.assertNotIn(namespace, existing_namespaces)

        seen_subnets = self.os_admin.subnets_client.list_subnets()
        seen_subnet_names = [n['name'] for n in seen_subnets['subnets']]
        self.assertNotIn(subnet, seen_subnet_names)

        seen_sgs = self.list_security_groups()
        seen_sg_ids = [sg['id'] for sg in seen_sgs]
        if net_crd['spec'].get('sgId', None):
            self.assertNotIn(net_crd['spec']['sgId'], seen_sg_ids)



# Copyright 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import json
import threading
import time
import uuid

import kubernetes
from oslo_log import log as logging
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from kuryr_tempest_plugin.tests.scenario import base
from kuryr_tempest_plugin.tests.scenario import consts

LOG = logging.getLogger(__name__)
CONF = config.CONF
TIMEOUT = 120


class TestHighAvailabilityScenario(base.BaseKuryrScenarioTest):

    @classmethod
    def skip_checks(cls):
        super(TestHighAvailabilityScenario, cls).skip_checks()
        if not (CONF.kuryr_kubernetes.ap_ha and
                CONF.kuryr_kubernetes.containerized):
            raise cls.skipException("kuryr-controller A/P HA must be enabled "
                                    "and kuryr-kubernetes must run in "
                                    "containerized mode.")

    def get_kuryr_leader_annotation(self):
        try:
            endpoint = self.k8s_client.CoreV1Api().read_namespaced_endpoints(
                consts.HA_ENDPOINT_NAME,
                CONF.kuryr_kubernetes.kube_system_namespace)
            annotation = endpoint.metadata.annotations[
                'control-plane.alpha.kubernetes.io/leader']
            return json.loads(annotation)
        except kubernetes.client.rest.ApiException:
            return None

    def wait_for_deployment_scale(self, desired_replicas,
                                  desired_state='Running'):
        def has_scaled():
            pods = self.k8s_client.CoreV1Api().list_namespaced_pod(
                CONF.kuryr_kubernetes.kube_system_namespace,
                label_selector='name=kuryr-controller')

            return (len(pods.items) == desired_replicas and
                    all([pod.status.phase == desired_state
                         for pod in pods.items]))

        self.assertTrue(test_utils.call_until_true(has_scaled, TIMEOUT, 5),
                        'Timed out waiting for deployment to scale')

    def scale_controller_deployment(self, replicas):
        self.k8s_client.AppsV1Api().patch_namespaced_deployment(
            'kuryr-controller', CONF.kuryr_kubernetes.kube_system_namespace,
            {'spec': {'replicas': replicas}})
        self.wait_for_deployment_scale(replicas)

    @decorators.idempotent_id('3f09e7d1-0897-46b1-ba9d-ea4116523025')
    def test_scale_up_controller(self):
        controller_deployment = (
            self.k8s_client.AppsV1Api().read_namespaced_deployment(
                CONF.kuryr_kubernetes.controller_deployment_name,
                CONF.kuryr_kubernetes.kube_system_namespace))

        # On cleanup scale to original number of replicas
        self.addCleanup(self.scale_controller_deployment,
                        controller_deployment.spec.replicas)

        # Scale to just a single replica
        self.scale_controller_deployment(1)

        # Create a pod and check connectivity
        self.create_and_ping_pod()

        # Get current leader annotation
        annotation = self.get_kuryr_leader_annotation()
        self.assertIsNotNone(annotation)
        transitions = annotation['leaderTransitions']

        # Scale the controller up and wait until it starts
        self.scale_controller_deployment(2)

        # Check if leader haven't switched
        annotation = self.get_kuryr_leader_annotation()
        self.assertEqual(transitions, annotation['leaderTransitions'])

        # Create another pod and check connectivity
        self.create_and_ping_pod()

    @decorators.idempotent_id('afe75fa5-e9ca-4f7d-bc16-8f1dd7884eea')
    def test_scale_down_controller(self):
        controller_deployment = (
            self.k8s_client.AppsV1Api().read_namespaced_deployment(
                CONF.kuryr_kubernetes.controller_deployment_name,
                CONF.kuryr_kubernetes.kube_system_namespace))

        # On cleanup scale to original number of replicas
        self.addCleanup(self.scale_controller_deployment,
                        controller_deployment.spec.replicas)

        # Scale to 2 replicas
        self.scale_controller_deployment(2)

        # Create a pod and check connectivity
        self.create_and_ping_pod()

        # Scale the controller down and wait until it stops
        self.scale_controller_deployment(1)

        # Create another pod and check connectivity
        self.create_and_ping_pod()

    @decorators.idempotent_id('3b218c11-c77b-40a8-ba09-5dd5ae0f8ae3')
    def test_auto_fencing(self):
        controller_deployment = (
            self.k8s_client.AppsV1Api().read_namespaced_deployment(
                CONF.kuryr_kubernetes.controller_deployment_name,
                CONF.kuryr_kubernetes.kube_system_namespace))

        # On cleanup scale to original number of replicas
        self.addCleanup(self.scale_controller_deployment,
                        controller_deployment.spec.replicas)

        # Scale to 2 replicas
        self.scale_controller_deployment(2)

        # Create a pod and check connectivity
        self.create_and_ping_pod()

        def hostile_takeover():
            """Malform endpoint annotation to takeover the leadership

            This method runs for 3 minutes and for that time it malforms the
            endpoint annotation to simulate another kuryr-controller taking
            over the leadership. This should make other kuryr-controllers to
            step down and stop processing any events for those 3 minutes.
            """
            timeout = datetime.datetime.utcnow() + datetime.timedelta(
                minutes=3)
            fake_name = str(uuid.uuid4())
            while datetime.datetime.utcnow() < timeout:
                current = datetime.datetime.utcnow()
                renew = current + datetime.timedelta(seconds=5)
                malformed = {
                    "holderIdentity": fake_name,
                    "leaseDurationSeconds": 5,
                    "acquireTime": current.strftime("%Y-%m-%dT%H:%M:%SZ"),
                    "renewTime": renew.strftime("%Y-%m-%dT%H:%M:%SZ"),
                    "leaderTransitions": 0,
                }
                self.k8s_client.CoreV1Api().patch_namespaced_endpoints(
                    consts.HA_ENDPOINT_NAME,
                    CONF.kuryr_kubernetes.kube_system_namespace,
                    {'metadata': {'annotations': {
                        'control-plane.alpha.kubernetes.io/leader':
                            json.dumps(malformed)}}})
                time.sleep(2)

        t = threading.Thread(target=hostile_takeover)
        t.start()

        # Create another pod and check that it's not getting wired.
        time.sleep(15)  # We need to wait a bit for controller to autofence.
        name, pod = self.create_pod(wait_for_status=False)

        def is_pod_running():
            pod_obj = self.k8s_client.CoreV1Api().read_namespaced_pod(
                name, 'default')

            return pod_obj.status.phase == 'Running'

        self.addCleanup(self.delete_pod, name)
        self.assertFalse(test_utils.call_until_true(is_pod_running, TIMEOUT,
                                                    5))

        # Wait 120 seconds more, malformed annotation should get cleared
        time.sleep(TIMEOUT)

        # Now pod should have the IP and be pingable
        ip = self.get_pod_ip(name)
        self.assertIsNotNone(ip)
        self.assertTrue(self.ping_ip_address(ip, ping_timeout=TIMEOUT))



# Copyright 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_log import log as logging
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from kuryr_tempest_plugin.tests import base

from oslo_config import cfg

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class PodTest(base.BaseAdminKuryrTest):

    def _list_pods(self):
        pods = self.k8s_client.list_pod_for_all_namespaces(watch=False)
        return pods

    def _delete_pod(self, pod_name, body=None, namespace='default'):
        if body is None:
            body = {}
        self.k8s_client.delete_namespaced_pod(name=pod_name,
                                              body=body,
                                              namespace=namespace)

    @decorators.idempotent_id('b6fbd21a-d7cb-497d-b03b-02e09cc2caf8')
    def test_create_list_pod(self):
        pod_name = data_utils.rand_name('pod')
        pod_manifest = {
            'apiVersion': 'v1',
            'kind': 'Pod',
            'metadata':
            {
                'name': pod_name
            },
            'spec': {
                'containers': [{
                    'image': 'busybox',
                    'name': 'sleep',
                    "args": [
                        "/bin/sh",
                        "-c",
                        "while true; do date; sleep 5; done"
                    ]
                }]
            }
        }
        self.k8s_client.create_namespaced_pod(body=pod_manifest,
                                              namespace='default')
        pod_names = [pod.metadata.name for pod in self._list_pods().items]
        self.assertIn(pod_name, pod_names)
        self.addCleanup(self._delete_pod, pod_name)
